#' @inherit Optimizer examples
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom checkmate assert_choice
#' @importFrom checkmate assert_flag
#' @importFrom checkmate assert_function
#' @importFrom checkmate assert_list
#' @importFrom checkmate assert_string
#' @importFrom checkmate check_character
#' @importFrom checkmate check_choice
#' @importFrom checkmate check_flag
#' @importFrom checkmate check_function
#' @importFrom checkmate check_integerish
#' @importFrom checkmate check_list
#' @importFrom checkmate check_names
#' @importFrom checkmate check_number
#' @importFrom checkmate check_string
#' @importFrom checkmate test_choice
#' @importFrom checkmate test_list
#' @importFrom checkmate test_number
#' @importFrom checkmate test_numeric
#' @importFrom checkmate test_r6
#' @importFrom checkmate test_scalar_na
#' @importFrom cli cat_bullet
#' @importFrom cli cli_abort
#' @importFrom cli cli_alert
#' @importFrom cli cli_alert_success
#' @importFrom cli cli_h1
#' @importFrom cli cli_h3
#' @importFrom cli cli_inform
#' @importFrom cli cli_warn
#' @importFrom lbfgsb3c lbfgsb3
#' @importFrom lbfgsb3c lbfgsb3c
#' @importFrom lbfgsb3c lbfgsb3f
#' @importFrom lbfgsb3c lbfgsb3x
#' @importFrom numDeriv grad
#' @importFrom numDeriv hessian
#' @importFrom oeli check_missing
#' @importFrom oeli check_numeric_vector
#' @importFrom oeli Dictionary
#' @importFrom oeli do.call_timed
#' @importFrom oeli function_arguments
#' @importFrom oeli function_body
#' @importFrom oeli function_defaults
#' @importFrom oeli input_check_response
#' @importFrom oeli merge_lists
#' @importFrom oeli test_numeric_vector
#' @importFrom oeli timed
#' @importFrom oeli variable_name
#' @importFrom pracma nelder_mead
#' @importFrom R6 R6Class
#' @importFrom stats nlm
#' @importFrom stats nlminb
#' @importFrom stats optim
#' @importFrom TestFunctions TF_ackley
#' @importFrom ucminf ucminf
## usethis namespace: end
NULL
